//---------------------------------------------------------------------------------------
//  FILE:    UIScreenListener_Armory_MainMenu_LWExpandedPerkTree
//  AUTHOR:  Amineri
//
//  PURPOSE: Implements hooks to add button to List in order to invoke Expanded Perk Tree for testing
//--------------------------------------------------------------------------------------- 

class UIScreenListener_Armory_MainMenu_LWExpandedPerkTree extends UIScreenListener;


var UIListItemString_SelfContained ExpandedListItem; // for integration into existing list
var UIArmory_MainMenu ParentScreen;

var delegate<OnItemSelectedCallback> NextOnSelectionChanged;

var localized string strExpandedMenuOption;
var localized string strExpandedTooltip;
var localized string ExpandedListItemDescription;

delegate OnItemSelectedCallback(UIList _list, int itemIndex);

// This event is triggered after a screen is initialized
event OnInit(UIScreen Screen)
{
	local UIPanel BG;

	if(class'XComGameState_LWPerkPackOptions'.static.IsBaseGamePerkUIEnabled()) return;

	ParentScreen = UIArmory_Mainmenu(Screen); 

	DisableSoldierIntros();

	//update mousewheel controls so that mousewheel moves scrollbar when over list
    if(ParentScreen != none)
    {
        BG = ParentScreen.Spawn(class'UIPanel', ParentScreen).InitPanel('armoryMenuBG');
        BG.bShouldPlayGenericUIAudioEvents = false;  
        BG.ProcessMouseEvents(ParentScreen.List.OnChildMouseEvent); // hook mousewheel to scroll MainMenu list instead of rotating soldier
 
 		NextOnSelectionChanged = ParentScreen.List.OnSelectionChanged;
		ParentScreen.List.OnSelectionChanged = OnSelectionChanged;

		AddListButton();
   }
}

//This event is triggered after a screen receives focus
event OnReceiveFocus(UIScreen Screen)
{

	ParentScreen = UIArmory_Mainmenu(Screen);
	AddListButton();
}

// This event is triggered after a screen loses focus
//event OnLoseFocus(UIScreen Screen);


// This event is triggered when a screen is removed
event OnRemoved(UIScreen Screen)
{
	ParentScreen = none;
}

//adds a button to the existing MainMenu list
simulated function AddListButton()
{
	ExpandedListItem = UIListItemString_SelfContained(ParentScreen.Spawn(class'UIListItemString_SelfContained', ParentScreen.List.ItemContainer).InitListItem(Caps(strExpandedMenuOption)).SetDisabled(false, strExpandedTooltip));
	ExpandedListItem.SetButtonBGClickHander (OnExpandedButtonCallback);
}

//callback handler for list button -- invokes the LW officer ability UI
simulated function OnExpandedButtonCallback(UIButton kButton)
{
	local XComHQPresentationLayer HQPres;
	local UIArmory_LWExpandedPromotion ExpandedScreen;

	HQPres = `HQPRES;
	ExpandedScreen = UIArmory_LWExpandedPromotion(HQPres.ScreenStack.Push(HQPres.Spawn(class'UIArmory_LWExpandedPromotion', HQPres), HQPres.Get3DMovie()));
	ExpandedScreen.InitPromotion(ParentScreen.GetUnitRef(), false);
}

simulated function OnSelectionChanged(UIList ContainerList, int ItemIndex)
{
	if (ContainerList.GetItem(ItemIndex) == ExpandedListitem) 
	{
		ParentScreen.MC.ChildSetString("descriptionText", "htmlText", class'UIUtilities_Text'.static.AddFontInfo(ExpandedListItemDescription, true));
		return;
	}
	NextOnSelectionChanged(ContainerList, ItemIndex);
}

static function DisableSoldierIntros()
{
	local XComGameState_HeadquartersXCom XComHQ;
	local XComGameState NewGameState;

	XComHQ = XComGameState_HeadquartersXCom(`XCOMHISTORY.GetSingleGameStateObjectForClass(class'XComGameState_HeadquartersXCom'));

	if(!XComHQ.bHasSeenFirstGrenadier)
	{
		NewGameState = class'XComGameStateContext_ChangeContainer'.static.CreateChangeState("Soldier Class Movie");
		XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
		NewGameState.AddStateObject(XComHQ);
		XComHQ.bHasSeenFirstGrenadier = true;
		`XCOMGAME.GameRuleset.SubmitGameState(NewGameState);
	}
	if(!XComHQ.bHasSeenFirstPsiOperative)
	{
		NewGameState = class'XComGameStateContext_ChangeContainer'.static.CreateChangeState("Soldier Class Movie");
		XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
		NewGameState.AddStateObject(XComHQ);
		XComHQ.bHasSeenFirstPsiOperative = true;
		`XCOMGAME.GameRuleset.SubmitGameState(NewGameState);
	}
	if(!XComHQ.bHasSeenFirstRanger)
	{
		NewGameState = class'XComGameStateContext_ChangeContainer'.static.CreateChangeState("Soldier Class Movie");
		XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
		NewGameState.AddStateObject(XComHQ);
		XComHQ.bHasSeenFirstRanger = true;
		`XCOMGAME.GameRuleset.SubmitGameState(NewGameState);
	}
	if(!XComHQ.bHasSeenFirstSharpshooter)
	{
		NewGameState = class'XComGameStateContext_ChangeContainer'.static.CreateChangeState("Soldier Class Movie");
		XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
		NewGameState.AddStateObject(XComHQ);
		XComHQ.bHasSeenFirstSharpshooter = true;
		`XCOMGAME.GameRuleset.SubmitGameState(NewGameState);
	}
	if(!XComHQ.bHasSeenFirstSpecialist)
	{
		NewGameState = class'XComGameStateContext_ChangeContainer'.static.CreateChangeState("Soldier Class Movie");
		XComHQ = XComGameState_HeadquartersXCom(NewGameState.CreateStateObject(class'XComGameState_HeadquartersXCom', XComHQ.ObjectID));
		NewGameState.AddStateObject(XComHQ);
		XComHQ.bHasSeenFirstSpecialist = true;
		`XCOMGAME.GameRuleset.SubmitGameState(NewGameState);
	}

}

defaultproperties
{
	// Leaving this assigned to none will cause every screen to trigger its signals on this class
	ScreenClass = UIArmory_MainMenu;
}