//---------------------------------------------------------------------------------------
//  FILE:    UIScreenListener_Armory_MainMenu_LW_LWExpandedPerkTree
//  AUTHOR:  Amineri
//
//  PURPOSE: Implements hooks to add button to List in order to invoke Expanded Perk Tree for testing
//--------------------------------------------------------------------------------------- 

class UIScreenListener_Armory_MainMenu_LW_LWExpandedPerkTree extends UIScreenListener;

`include(LW_PerkPack\Src\LW_PerkPack.uci)


var UIListItemString ExpandedListItem; // for integration into existing list
var UIArmory_MainMenu_LW ParentScreen;

var delegate<OnItemSelectedCallback> NextOnSelectionChanged;

var localized string strExpandedMenuOption;
var localized string strExpandedTooltip;
var localized string ExpandedListItemDescription;

delegate OnItemSelectedCallback(UIList _list, int itemIndex);

// This event is triggered after a screen is initialized
event OnInit(UIScreen Screen)
{
	local UIPanel BG;

	ParentScreen = UIArmory_MainMenu_LW(Screen); 

	//update mousewheel controls so that mousewheel moves scrollbar when over list
    if(ParentScreen != none)
    {
        BG = ParentScreen.Spawn(class'UIPanel', ParentScreen).InitPanel('armoryMenuBG');
        BG.bShouldPlayGenericUIAudioEvents = false;  
        BG.ProcessMouseEvents(ParentScreen.List.OnChildMouseEvent); // hook mousewheel to scroll MainMenu list instead of rotating soldier
 
		if(!ParentScreen.default.bUse2WideAbilityTree && !class'XComGameState_LWPerkPackOptions'.static.IsBaseGamePerkUIEnabled())
		{
			class'UIScreenListener_Armory_MainMenu_LWExpandedPerkTree'.static.DisableSoldierIntros();
			ParentScreen.PromotionButton.ButtonBG.OnClickedDelegate = OnExpandedButtonCallback;
 			//NextOnSelectionChanged = ParentScreen.List.OnSelectionChanged;
			//ParentScreen.List.OnSelectionChanged = OnSelectionChanged;

			//AddListButton();
   			//UpdateListButton();
		}
	}
}

//This event is triggered after a screen receives focus
event OnReceiveFocus(UIScreen Screen)
{

	ParentScreen = UIArmory_MainMenu_LW(Screen);
	if(!ParentScreen.default.bUse2WideAbilityTree)
	{
		ParentScreen.PromotionButton.ButtonBG.OnClickedDelegate = OnExpandedButtonCallback;
	}
	//AddListButton();
	//UpdateListButton();
}

// This event is triggered after a screen loses focus
//event OnLoseFocus(UIScreen Screen);


// This event is triggered when a screen is removed
event OnRemoved(UIScreen Screen)
{
	ParentScreen = none;
}

//adds a button to the existing MainMenu list
simulated function AddListButton()
{
	ExpandedListItem = ParentScreen.Spawn(class'UIListItemString', ParentScreen.List.ItemContainer).InitListItem(); //Caps(strExpandedMenuOption));
	//ExpandedListItem.SetDisabled(false, strExpandedTooltip));
	ExpandedListItem.ButtonBG.OnClickedDelegate = OnExpandedButtonCallback;
}

simulated function UpdateListButton()
{
	local XComGameState_Unit Unit;
	local string PromoteIcon;

	Unit = XComGameState_Unit(`XCOMHISTORY.GetGameStateForObjectID(ParentScreen.UnitReference.ObjectID));
	`PPDEBUG("Screen Listener (UIArmory_MainMenu_LW): UnitName=" $ Unit.GetName(eNameType_FullNick));
	if(Unit.ShowPromoteIcon())
	{
		PromoteIcon = class'UIUtilities_Text'.static.InjectImage(class'UIUtilities_Image'.const.HTML_PromotionIcon, 20, 20, 0) $ " ";
		ExpandedListItem.SetText(PromoteIcon $ ParentScreen.default.m_strPromote);
	}
	else
	{
		ExpandedListItem.SetText(ParentScreen.default.m_strAbilities);
	}
	if(Unit.GetRank() < 1 && !Unit.CanRankUpSoldier())
	{
		ExpandedListItem.SetDisabled(true, ParentScreen.default.m_strRookiePromoteTooltip);
	}
}

//callback handler for list button -- invokes the LW officer ability UI
simulated function OnExpandedButtonCallback(UIButton kButton)
{
	local XComHQPresentationLayer HQPres;
	local UIArmory_LWExpandedPromotion ExpandedScreen;
	local UIArmory_Promotion PromotionScreen;
	local XComGameState_Unit UnitState;
	local name SoldierClassName;
	local StateObjectReference UnitRef;

	if(ParentScreen.CheckForDisabledListItem(kButton)) return;

	HQPres = `HQPRES;
	UnitRef = ParentScreen.GetUnitRef();
	UnitState = XComGameState_Unit(`XCOMHISTORY.GetGameStateForObjectID(UnitRef.ObjectID));
	SoldierClassName = class'X2StrategyGameRulesetDataStructures'.static.PromoteSoldier(UnitRef);
	if (SoldierClassName == '')
	{
		SoldierClassName = UnitState.GetSoldierClassTemplate().DataName;
	}
	
	if(UnitState == none)
	{
		`REDSCREEN("LW PerkPack: Promotion UI callback without a valid Unit State.");
		return;
	}

	// The ShowPromotionUI will get triggered at the end of the class movie if it plays, or...
	//if (!class'X2StrategyGameRulesetDataStructures'.static.ShowClassMovie(SoldierClassName, UnitRef))
	//{
		// ...this wasn't the first time we saw this unit's new class so just show the UI
	if (UnitState.GetSoldierClassTemplateName() == 'PsiOperative')
	{
		PromotionScreen = UIArmory_PromotionPsiOp(HQPres.ScreenStack.Push(HQPres.Spawn(class'UIArmory_PromotionPsiOp', HQPres), HQPres.Get3DMovie()));
		PromotionScreen.InitPromotion(UnitRef, false);
	}
	else
	{
		ExpandedScreen = UIArmory_LWExpandedPromotion(HQPres.ScreenStack.Push(HQPres.Spawn(class'UIArmory_LWExpandedPromotion', HQPres), HQPres.Get3DMovie()));
		ExpandedScreen.InitPromotion(UnitRef, false);
	}
	PromotionScreen = ExpandedScreen;
	`XEVENTMGR.TriggerEvent('OnUnitPromotion', UnitState, PromotionScreen); // trigger for manual promotion events -- typically used for first-time promotion dialogue boxes, cinematics, etc

}

simulated function OnSelectionChanged(UIList ContainerList, int ItemIndex)
{
	if (ContainerList.GetItem(ItemIndex) == ExpandedListitem) 
	{
		ParentScreen.MC.ChildSetString("descriptionText", "htmlText", class'UIUtilities_Text'.static.AddFontInfo(ExpandedListItemDescription, true));
		return;
	}
	NextOnSelectionChanged(ContainerList, ItemIndex);
}

defaultproperties
{
	// Leaving this assigned to none will cause every screen to trigger its signals on this class
	ScreenClass = UIArmory_MainMenu_LW;
}