//---------------------------------------------------------------------------------------
//  FILE:    X2Ability_RequiredToHitChance.uc
//  AUTHOR:  Amineri (Long War Studios)
//  PURPOSE: Defines a new condition in which some minimal to-hit chance is required
//--------------------------------------------------------------------------------------- 
class X2Condition_RequiredToHitChance extends X2Condition;

`include(LW_PerkPack\Src\LW_PerkPack.uci)

var int MinimumRequiredHitChance;
var array<name> ExcludedAbilities; // if target has any of these abilities, then the minimum requirement is ignored

event name CallAbilityMeetsCondition(XComGameState_Ability kAbility, XComGameState_BaseObject kTarget)
{
	local XComGameStateHistory		History;
	local X2AbilityTemplate			AbilityTemplate;
	local X2AbilityToHitCalc		ToHitCalc;
	local int						HitChance;
	local AvailableTarget			Target;
	local StateObjectReference		AbilityRef;
	local XComGameState_Unit		kTargetUnit;
	local XComGameState_Ability		Ability;
	local name						AbilityName;

	History = `XCOMHISTORY;
	kTargetUnit = XComGameState_Unit(kTarget);
	foreach ExcludedAbilities(AbilityName)
	{
		if (kTargetUnit != none)
		{
			AbilityRef = kTargetunit.FindAbility(AbilityName); //'LightningReflexes_LW');
			Ability = XComGameState_Ability(History.GetGameStateForObjectID(AbilityRef.ObjectID));
			if (Ability != none)
				return 'AA_Success'; 
		}
	}
	AbilityTemplate = kAbility.GetMyTemplate();
	
	//`PPDEBUG("RequiredToHitChance: Testing");

	ToHitCalc = AbilityTemplate.AbilityToHitCalc;	
	if(ToHitCalc != none)
	{
		Target.PrimaryTarget = kTarget.GetReference();
		HitChance = ToHitCalc.GetShotBreakdown(kAbility, Target);
		//`PPDEBUG("RequiredToHitChance: CurrentHitChance = " $ HitChance);
		if(HitChance < MinimumRequiredHitChance)
		{
			return 'AA_NotInRange';
		}
	}
	return 'AA_Success'; 
}
