//---------------------------------------------------------------------------------------
//  FILE:    X2Effect_CloseEncounters
//  AUTHOR:  John Lumpkin (Long War Studios)
//  PURPOSE: Grants action under certain conditions
//--------------------------------------------------------------------------------------- 

class X2Effect_CloseEncounters extends X2Effect_Persistent config (LW_PerkPack);

var config int CE_USES_PER_TURN;
var config array<name> CE_ABILITYNAMES;
var config int CE_MAX_TILES;

simulated protected function OnEffectAdded(const out EffectAppliedData ApplyEffectParameters, XComGameState_BaseObject kNewTargetState, XComGameState NewGameState, XComGameState_Effect NewEffectState)
{
	local XComGameState_Effect_EffectCounter	CEEffectState;
	local X2EventManager						EventMgr;
	local Object								ListenerObj, EffectObj;
	local XComGameState_Unit					UnitState;

	EventMgr = `XEVENTMGR;
	EffectObj = NewEffectState;
	UnitState = XComGameState_Unit(`XCOMHISTORY.GetGameStateForObjectID(NewEffectState.ApplyEffectParameters.SourceStateObjectRef.ObjectID));

	if (GetCECounter(NewEffectState) == none)
	{
		CEEffectState = XComGameState_Effect_EffectCounter(NewGameState.CreateStateObject(class'XComGameState_Effect_EffectCounter'));
		CEEffectState.InitComponent();
		NewEffectState.AddComponentObject(CEEffectState);
		NewGameState.AddStateObject(CEEffectState);
	}
	ListenerObj = CEEffectState;
	if (ListenerObj == none)
	{
		`Redscreen("CE: Failed to find CE Component when registering listener");
		return;
	}
    EventMgr.RegisterForEvent(ListenerObj, 'PlayerTurnBegun', CEEffectState.ResetUses, ELD_OnStateSubmitted);
	EventMgr.RegisterForEvent(EffectObj, 'CloseEncounters', NewEffectState.TriggerAbilityFlyover, ELD_OnStateSubmitted, , UnitState);
}

simulated function OnEffectRemoved(const out EffectAppliedData ApplyEffectParameters, XComGameState NewGameState, bool bCleansed, XComGameState_Effect RemovedEffectState)
{
	local XComGameState_BaseObject EffectComponent;
	local Object EffectComponentObj;
	
	super.OnEffectRemoved(ApplyEffectParameters, NewGameState, bCleansed, RemovedEffectState);

	EffectComponent = GetCECounter(RemovedEffectState);
	if (EffectComponent == none)
		return;

	EffectComponentObj = EffectComponent;
	`XEVENTMGR.UnRegisterFromAllEvents(EffectComponentObj);

	NewGameState.RemoveStateObject(EffectComponent.ObjectID);
}

static function XComGameState_Effect_EffectCounter GetCECounter(XComGameState_Effect Effect)
{
	if (Effect != none) 
		return XComGameState_Effect_EffectCounter(Effect.FindComponentObject(class'XComGameState_Effect_EffectCounter'));
	return none;
}

function bool PostAbilityCostPaid(XComGameState_Effect EffectState, XComGameStateContext_Ability AbilityContext, XComGameState_Ability kAbility, XComGameState_Unit SourceUnit, XComGameState_Item AffectWeapon, XComGameState NewGameState, const array<name> PreCostActionPoints, const array<name> PreCostReservePoints)
{
	local XComGameState_Effect_EffectCounter	CurrentCECounter, UpdatedCECounter;
	local X2EventManager						EventMgr;
	local XComGameState_Ability					AbilityState;
	local XComGameState_Unit					TargetUnit;

	if (SourceUnit.IsUnitAffectedByEffectName(class'X2Effect_Serial'.default.EffectName))
		return false;

	if (SourceUnit.IsUnitAffectedByEffectName(class'X2Effect_DeathfromAbove'.default.EffectName))
		return false;

	if (PreCostActionPoints.Find('RunAndGun') != -1)
		return false;

	if (kAbility == none)
		return false;

	if (kAbility.SourceWeapon != EffectState.ApplyEffectParameters.ItemStateObjectRef)
		return false;

	CurrentCECounter = GetCECounter(EffectState);
	If (CurrentCECounter != none)	 
	{
		if (CurrentCECounter.uses >= default.CE_USES_PER_TURN)		
			return false;
	}
	TargetUnit = XComGameState_Unit(NewGameState.GetGameStateForObjectID(AbilityContext.InputContext.PrimaryTarget.ObjectID));  	

	if (TargetUnit == none)
		return false;

	//`LOG (string (SourceUnit.TileDistanceBetween(TargetUnit)));
	//`LOG (string (default.CE_MAX_TILES));

	if (SourceUnit.TileDistanceBetween(TargetUnit) > default.CE_MAX_TILES + 1)
		return false;

	//`LOG ("CE7");

	if (XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(EffectState.ApplyEffectParameters.AbilityStateObjectRef.ObjectID)) == none)
		return false;

	//`LOG ("CE8");

	AbilityState = XComGameState_Ability(`XCOMHISTORY.GetGameStateForObjectID(EffectState.ApplyEffectParameters.AbilityStateObjectRef.ObjectID));

	if (AbilityState != none)
	{
		if (default.CE_ABILITYNAMES.Find(kAbility.GetMyTemplateName()) != -1)
		{
			//`LOG ("CE9");
			
			if (SourceUnit.NumActionPoints() < 2 && PreCostActionPoints.Length > 0)
			{
				//`LOG ("CE10");

				UpdatedCECounter = XComGameState_Effect_EffectCounter(NewGameState.CreateStateObject(class'XComGameState_Effect_EffectCounter', CurrentCECounter.ObjectID));
				SourceUnit.ActionPoints.AddItem(class'X2CharacterTemplateManager'.default.StandardActionPoint);
				UpdatedCECounter.uses += 1;
				NewGameState.AddStateObject(UpdatedCECounter);
				NewGameState.AddStateObject(SourceUnit);
				EventMgr = `XEVENTMGR;
				EventMgr.TriggerEvent('CloseEncounters', AbilityState, SourceUnit, NewGameState);
				return true;
			}
		}
	}
	return false;
}