///---------------------------------------------------------------------------------------
//  FILE:    X2Effect_FieldSurgeon
//  AUTHOR:  Amineri (Long War Studios)
//  PURPOSE: Implements effect for FieldSurgeon ability -- this allows healing of lowest hp at end of mission
//--------------------------------------------------------------------------------------- 
//---------------------------------------------------------------------------------------
class X2Effect_FieldSurgeon extends X2Effect_Persistent config(LW_PerkPack);

`include(LW_PerkPack\Src\LW_PerkPack.uci)

var config array<int> FIELD_SURGEON_CHANCE_FOR_NUM_EFFECTS;
var name FieldSurgeonAppliedUnitValue;

function UnitEndedTacticalPlay(XComGameState_Effect EffectState, XComGameState_Unit UnitState)
{
	local XComGameStateHistory		History;
//	local XComGameStateContext_ChangeContainer ChangeContainer;
//	local XComGameState				GameState;
	local XComGameState_Unit		SourceUnitState; //, UpdatedTargetState;
	local int						NumEffects;
	local UnitValue					AppliedFSValue;
	local bool						bApplyFieldSurgeon;

	History = `XCOMHISTORY;
	SourceUnitState = XComGameState_Unit(History.GetGameStateForObjectID(EffectState.ApplyEffectParameters.SourceStateObjectRef.ObjectID));

	`PPTRACE("Field Surgeon: TargetUnit=" $ UnitState.GetFullName() $ ", SourceUnit=" $ SourceUnitState.GetFullName());

	if(!FieldSurgeonEffectIsValidForSource(SourceUnitState)) { return; }

	`PPTRACE("Field Surgeon: Source Unit Valid.");

	if(UnitState == none) { return; }
	if(UnitState.IsDead()) { return; }
	if(UnitState.IsBleedingOut()) { return; }
	if(!CanBeHealed(UnitState)) { return; }

	`PPTRACE("Field Surgeon: Target Unit Can Be Healed.");

	bApplyFieldSurgeon = false;
	NumEffects = 0;
	if(UnitState.GetUnitValue(default.FieldSurgeonAppliedUnitValue, AppliedFSValue))
	{
		`PPTRACE("Field Surgeon: FSValue Found, Value=" $ int(AppliedFSValue.fValue));
		NumEffects = int(AppliedFSValue.fValue);
		if(NumEffects >= default.FIELD_SURGEON_CHANCE_FOR_NUM_EFFECTS.Length)
			NumEffects = default.FIELD_SURGEON_CHANCE_FOR_NUM_EFFECTS.Length-1;
	}
	`PPTRACE("Field Surgeon: NumEffects=" $ NumEffects $ ", Chance=" $ default.FIELD_SURGEON_CHANCE_FOR_NUM_EFFECTS[NumEffects]);
	if(`SYNC_RAND(100) < default.FIELD_SURGEON_CHANCE_FOR_NUM_EFFECTS[NumEffects])
		bApplyFieldSurgeon = true;

	`PPTRACE("Field Surgeon: ApplyFieldSurgeon=" $ bApplyFieldSurgeon);

	if(bApplyFieldSurgeon)
	{
		`PPTRACE("Field Surgeon : Pre update LowestHP=" $ UnitState.LowestHP);
		UnitState.LowestHP += 1;
		`PPTRACE("Field Surgeon : Post update LowestHP=" $ UnitState.LowestHP);
		UnitState.ModifyCurrentStat(eStat_HP, 1);
		UnitState.SetUnitFloatValue(default.FieldSurgeonAppliedUnitValue, AppliedFSValue.fValue + 1, eCleanup_BeginTactical);
	}

	super.UnitEndedTacticalPlay(EffectState, UnitState);
}

function bool CanBeHealed(XComGameState_Unit UnitState)
{
	 return (UnitState.LowestHP < UnitState.GetMaxStat(eStat_HP) && UnitState.LowestHP > 0);
}

function bool FieldSurgeonEffectIsValidForSource(XComGameState_Unit SourceUnit)
{
	if(SourceUnit == none) { return false; }
	if(SourceUnit.IsDead()) { return false; }
	if(SourceUnit.IsBleedingOut()) { return false; }
	if(SourceUnit.bCaptured) { return false; }
	if(SourceUnit.LowestHP == 0) { return false; }
	if(SourceUnit.IsUnconscious()) { return false; }
	return true;
}

DefaultProperties
{
	EffectName="FieldSurgeon"
	DuplicateResponse=eDupe_Allow
	FieldSurgeonAppliedUnitValue="FieldSurgeonHealed"
}