//--------------------------------------------------------------------------------------- 
//  FILE:    X2Effect_LockdownDamage
//  AUTHOR:  John Lumpkin (Long War Studios)
//  PURPOSE: Sets up damage bonuses for Lockdown
//---------------------------------------------------------------------------------------
class X2Effect_LockdownDamage extends X2Effect_Persistent config (LW_PerkPack);

`include(LW_PerkPack\Src\LW_PerkPack.uci)

var config float LOCKDOWN_DAMAGE_BONUS_PCT;
var config int LOCKDOWN_TOHIT_BONUS;

function int GetAttackingDamageModifier(XComGameState_Effect EffectState, XComGameState_Unit Attacker, Damageable TargetDamageable, XComGameState_Ability AbilityState, const out EffectAppliedData AppliedData, const int CurrentDamage, optional XComGameState NewGameState)
{
    local XComGameState_Item SourceWeapon;
    local XComGameState_Unit TargetUnit;

	if(AbilityState.GetMyTemplateName() == 'SuppressionShot_LW' || AbilityState.GetMyTemplateName() == 'AreaSuppressionShot_LW')
	{
		if(AppliedData.AbilityResultContext.HitResult == eHit_Success)
		{
			SourceWeapon = AbilityState.GetSourceWeapon();
			if(SourceWeapon != none) 
			{
				TargetUnit = XComGameState_Unit(TargetDamageable);
				if(TargetUnit != none)
				{
					if (Attacker.HasSoldierAbility('Lockdown'))
					{
						`PPTRACE ("LD6");
						return int (CurrentDamage * (default.LOCKDOWN_DAMAGE_BONUS_PCT / 100));
					}
				}
            }
        }
    }
    return 0;
}

function GetToHitModifiers(XComGameState_Effect EffectState, XComGameState_Unit Attacker, XComGameState_Unit Target, XComGameState_Ability AbilityState, class<X2AbilityToHitCalc> ToHitType, bool bMelee, bool bFlanking, bool bIndirectFire, out array<ShotModifierInfo> ShotModifiers)
{
    local XComGameState_Item SourceWeapon;
    local ShotModifierInfo ShotInfo;

	if(AbilityState.GetMyTemplateName() == 'SuppressionShot_LW' || AbilityState.GetMyTemplateName() == 'AreaSuppressionShot_LW')
	{
		SourceWeapon = AbilityState.GetSourceWeapon();    
		if (SourceWeapon != none && Target != none)
		{
			if (Attacker.HasSoldierAbility('Lockdown'))
			{
				ShotInfo.ModType = eHit_Success;
				ShotInfo.Reason = FriendlyName;
				ShotInfo.Value = default.LOCKDOWN_TOHIT_BONUS;
				ShotModifiers.AddItem(ShotInfo);
			}
        }
    }    
}
